package sudoku;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.FileNotFoundException;

/** Helpers class for parsing Sudoku files */
public class SudokuParser {
    private static final int BOARD_SIZE = 9;

    // * Empty private constructor */
    private SudokuParser() {
        // Empty
    };

    /**
     * Parses a Sudoku file and returns a 2D array of integers
     * 
     * @param filePath Path to the Sudoku file
     * @return 2D array of integers representing the Sudoku board
     * @throws IOException              If an IO error occurs
     * @throws FileNotFoundException    When the file cannot be found
     * @throws NumberFormatException    When the file contains invalid characters
     * @throws IllegalArgumentException When the file contains an invalid number of
     *                                  rows or columns
     */
    public static int[][] parseSudoku(String filePath)
            throws IOException, FileNotFoundException, NumberFormatException, IllegalArgumentException {
        int[][] sudokuBoard = new int[BOARD_SIZE][BOARD_SIZE];

        // In practice we could just split the entire file into a single string and then
        // parse it into an array of integers, which is then partitioned into a 2D
        // array.
        // However, this is how the assignment is specified, so we will do it this way.

        // Try to read the file with a BufferedReader
        try (BufferedReader reader = new BufferedReader(new FileReader(filePath))) {
            String line;
            int row = 0;

            // While there are lines to read and we haven't reached the end of the board
            while ((line = reader.readLine()) != null && row < BOARD_SIZE) {
                // Split it into an array of strings
                String[] values = line.trim().split("\\s+");

                // Check that the number of columns is correct
                if (values.length != BOARD_SIZE) {
                    throw new IllegalArgumentException("Invalid number of columns in the Sudoku file.");
                }

                // Parse the strings into integers and add them to the board
                for (int col = 0; col < BOARD_SIZE; col++) {
                    sudokuBoard[row][col] = Integer.parseInt(values[col]); // Throws NumberFormatException
                }

                row++;
            }

            if (row != BOARD_SIZE) {
                throw new IllegalArgumentException("Invalid number of rows in the Sudoku file.");
            }
        } catch (FileNotFoundException e) {
            throw new FileNotFoundException("The Sudoku file could not be found.");
        } catch (IOException e) {
            throw new IOException("An error occurred while reading the Sudoku file.");
        } catch (NumberFormatException e) {
            throw new NumberFormatException("The Sudoku file contains invalid characters.");
        }

        return sudokuBoard;
    }
}
